/*
 * SceneGraph.java
 *
 * Tree of SceneObjects used to maintain scene heirarchy.
 *
 * 2012 Brandon Reiss. All Rights Reserved. Do not duplicate or distribute
 * without the author's consent.
*/
package com.brandonreiss.geometry;

import com.brandonreiss.math.*;

///<summaryA tree of scene objects to render.</summary>
public class SceneGraph extends SceneObject {
  ///<summary>Default root transform.</summary>
  public Matrix4x4 root = Matrix4x4.createMatrixIdentity();
  ///<summary>Matrix stack to optimize transform computation.</summary>
  private MatrixStack matrixStack = new MatrixStack();

  ///<summary>Update geometry transforms.</summary>
  private void updateTransform(SceneObject t) {
    // Push the root local transform.
    matrixStack.push(t.transformLocal);
    {
      // Set the global transform.
      t.transform.set(matrixStack.top());
      t.onTransformUpdated();
      // Update the children.
      for (SceneObject c : t.children) {
        updateTransform(c);
      }
    }
    matrixStack.pop();
  }

  ///<summary>Update geometry transforms.</summary>
  public void updateTransform() {
    // Push the root transform (always identity).
    matrixStack.push(root);
    {
      // Traverse scene graph and transform it.
      for (SceneObject c : children) {
        updateTransform(c);
      }
    }
    matrixStack.pop();
  }
}

